function [result,ee, stat,Tri, Vertex,VI,EE]=CubatureTriang(F,Tri,Vertex,...
    VI,EE,opt,varargin)
%CUBATTRI - cubature on a collection of triangles
%call [result,ee, stat,Tri, Vertex,VI,EE]=CubatTri2(F,Tri,Vertex,...
%    VI,EE,opt,varargin)
%F - function to be integrated
%Tri - collection of triangles
%Vertex - collection of vertices
%VI - value of integral for a triangle
%EE - error estimation for a triangle
%opt - options
% AbsoluteError - default 1e-3;
% RelativeError - default 0;
% Trace - tracing indicator, default 'off';
% InitialFunction - initialization function; return cubature parameters:
%        weights, nodes, null rulles type
%        call [W,G,m,p]=initf
%        default @BerntsenEspelid;
% MaxFctEval - no function evaluations, default 1e5;
% Restart - restart flag, default 'no';
%result - approx. of integral
%ee - error estimation
%stat - statistics

global FEN TRIN nH Heap nT nV
Heap=[]; nH=0;
FEN=0;  sfflag='success';
na=6; 
if nargin<na %no options
   opt=optset(); 
end
initf=opt.InitialFunction;
if strcmp(opt.Restart,'yes')==1
    restart=1;
else
    restart=0;
end
if strcmp(opt.Trace,'on')==1
    trace=1;
else
    trace=0;
end
errrel=opt.RelativeError;
errabs=opt.AbsoluteError;
NmaxFev=opt.MaxFctEval;
sfl=(nargout>=3);
[w,xi,m,p]=initf(); %init elem. formula
nevstep=1+3*m+6*p;
%initialization of data structures
[nT,m1]=size(Tri);
[m2,nV]=size(Vertex);
TRIN=nT;
if trace
    xt=[]; %clf; 
    triplot(Tri,Vertex(1,:),Vertex(2,:));
    hold on
end

%initialization of VI and EE
for k=1:nT
    if restart==0
        V=Vertex(:,Tri(k,:));
        [VI(k),EE(k),xtrace]=fselcub(F,V,w,xi,m,p,varargin{:});
        if trace, xt=[xt,xtrace]; end
        FEN=FEN+nevstep;
    end
    InsertIntoHeap(EE,k);
end
result=sum(VI(1:nT));
ee=sum(EE(1:nT));

%the main loop
while (ee>errabs) & (ee>errrel*abs(result))
    %adjust result, heap and ee
    top=ExtractMaxFromHeap(EE);
    cT=Tri(top,:); %current triangle
    result=result-VI(top);
    ee=ee-EE(top);
    %save vertices
    cV=Vertex(:,Tri(top,:)); %vertices of current triangle
    %subdivision - new vertices
    M1=(cV(:,2)+cV(:,3))/2;
    M2=(cV(:,1)+cV(:,3))/2;
    M3=(cV(:,2)+cV(:,1))/2;
    [Vertex,iv1,nV]=NewVertex(Vertex,nV,M1);
    [Vertex,iv2,nV]=NewVertex(Vertex,nV,M2);
    [Vertex,iv3,nV]=NewVertex(Vertex,nV,M3);
    %subdivision -new triangles
    it1=[cT(1),iv2,iv3]; %this replace current triangle
    Tri(top,:)=it1;
    V=Vertex(:,Tri(top,:));
    [VI(top),EE(top),xtrace]=fselcub(F,V,w,xi,m,p,varargin{:});
    if trace, xt=[xt,xtrace]; end
    FEN=FEN+nevstep;
    InsertIntoHeap(EE,top);
    it2=[cT(2),iv3,iv1];
    [Tri,iit2,nT]=NewTriangle(Tri,nT,it2);
    V=Vertex(:,Tri(iit2,:));
    [VI(iit2),EE(iit2),xtrace]=fselcub(F,V,w,xi,m,p,varargin{:});
    if trace, xt=[xt,xtrace]; end
    FEN=FEN+nevstep;
    InsertIntoHeap(EE,iit2);
    it3=[cT(3),iv1,iv2];
    [Tri,iit3,nT]=NewTriangle(Tri,nT,it3);
    V=Vertex(:,Tri(iit3,:));
    [VI(iit3),EE(iit3),xtrace]=fselcub(F,V,w,xi,m,p,varargin{:});
    if trace, xt=[xt,xtrace]; end
    FEN=FEN+nevstep;
    InsertIntoHeap(EE,iit3);
    it4=[iv1,iv2,iv3];
    [Tri,iit4,nT]=NewTriangle(Tri,nT,it4);
    V=Vertex(:,Tri(iit4,:));
    [VI(iit4),EE(iit4),xtrace]=fselcub(F,V,w,xi,m,p,varargin{:});
    if trace, xt=[xt,xtrace]; end
    FEN=FEN+nevstep;
    InsertIntoHeap(EE,iit4);
    TRIN=TRIN+3;
    iit1=top;
    result=result+VI(iit1)+VI(iit2)+VI(iit3)+VI(iit4);
    ee=ee+EE(iit1)+EE(iit2)+EE(iit3)+EE(iit4);
    if FEN > NmaxFev
        s=sprintf('two many function evaluation %d',FEN);
        warning(s);
        sfflag='fail';
        break
    end
end
if sfl
    stat=struct('nev',FEN,'ntri',TRIN,'finalres',sfflag);
end
%adjust data structures before return
Tri=Tri(1:nT,:);
Vertex=Vertex(:,1:nV);
VI=VI(1:nT);
EE=EE(1:nT);
if trace & ~isempty(xt), tracefun(xt); end

function tracefun(L)
%TRACEFUN - represents points where function is evaluated
plot(L(1,:),L(2,:),'.k','Markersize',4);
hold off
