function [vi,ere,xtrace]=fselcub(F,V,w,xi,m,p,varargin)
%FSELCUB - elementary cubature on triagle -fully symmetric form
%call [vi,ere,xtrace]=fselcub(F,V,w,xi,m,p,varargin)
%input
%F - integrand
%V - vertices
%w -weights of cubature formulas and null rules
%xi - nodes in barycentric coordinates
%m - # nodes with multiplicity 3
%p - # nodes with multiplicity 6
%output
%vi - value  of integrand
%ere - error estimation
%xtrace - points where f is evaluated (useful for tracing)

[wm,wn]=size(w);
NULL=zeros(wm-1,1);
[xm,xn]=size(xi);
xin=[xi;ones(1,xn)-xi(1,:)-xi(2,:)];
xtrace=[];
%area
area=abs(V(1,1)*V(2,2)-V(1,2)*V(2,1)-V(1,1)*V(2,3)+...
    V(1,3)*V(2,1)+V(1,2)*V(2,3)-V(1,3)*V(2,2))/2;

%contribution from the center
X=[sum(V(1,:))/3;sum(V(2,:))/3];
fc=F(X,varargin{:});
xtrace=[xtrace,X];
bv=w(:,1)*fc;
%contribution from the points with multiplicity 3
per3=[1,2,3;2,3,1;3,1,2];
for i=2:2+m-1
    u=xin(:,i);
    X=V*u(per3);
    xtrace=[xtrace,X];
    bv=bv+sum(w(:,i)*F(X,varargin{:}),2);
end
%contribution from the points with multiplicity 6
per6=[1,2,3;1,3,2;2,1,3;2,3,1;3,1,2;3,2,1]';
for i=2+m:2+m+p-1
    u=xin(:,i);
    X=V*u(per6);
    xtrace=[xtrace,X];
    bv=bv+sum(w(:,i)*F(X,varargin{:}),2);
end
vi=area*bv(1);
%error estimation
NULL=bv(2:end);
Nm=length(NULL);
if Nm==1
    NULL=area*NULL;
    ere=abs(vi-NULL);
else
    NULL=reshape(NULL,2,Nm/2);
    Nm=length(NULL);
    E=area*sqrt(sum(NULL.^2));
    for j=1:Nm-1
        if E(j+1)~=0
            R(j)=E(j)/E(j+1);
        else
            R(j)=1;
        end
    end
    r=max(R);
    %decision
    if r>=1 %nonasymptotic
        ere=10*max(E);
    elseif r >= 1/2  %weakly-asymptotic
        ere=10*r*E(1);
    else %strongly assymptotic
        ere=40*r^3*E(1);
    end %if
    noise=abs(vi)*50*eps;
    if (E(1)<=noise) & (E(2)<=noise)
        ere=noise;
    end
    ere=max(noise,ere);
end
        
        