function options=optset(varargin)


% Print out possible values of properties.
if (nargin == 0) && (nargout == 0)
    fprintf('   AbsoluteError: [ positive scalar or vector {1e-3} ]\n');
    fprintf(' InitialFunction: [ function handler {@BerntsenEspelid} ]\n');
    fprintf('      MaxFctEval: [ integer {100000}]\n');
    fprintf('   RelativeError: [ nonnegative scalar {0} ]\n');
    fprintf('         Restart: [ yes| {no} ]\n');
    fprintf('           Trace: [ on | {off} ]\n');
    fprintf('           Subdivision: [ flexible | {congruent} ]\n');
    return;
end

%defaults
options.AbsoluteError=1e-3;
options.RelativeError=0;
options.Trace='off';
options.InitialFunction=@BerntsenEspelid;
options.MaxFctEval=1e5;
options.Restart='no';
options.Subdivision='congruent';
if (nargin == 0) && (nargout ~= 0)
    return
end
Names = [
    'AbsoluteError  '
    'InitialFunction'
    'MaxFctEval     '
    'RelativeError  '
    'Restart        '
    'Trace          '
    'Subdivision    '
    ];
m = size(Names,1);
names = lower(Names);

% Combine all leading options structures o1, o2, ... in odeset(o1,o2,...).
%options = [];
%for j = 1:m
%    options.(deblank(Names(j,:))) = [];
%end

i = 1;
while i <= nargin
    arg = varargin{i};
    if ischar(arg)                         % arg is an option name
        break;
    end
    if ~isempty(arg)                      % [] is a valid options argument
        if ~isa(arg,'struct')
            error('optset:NoPropNameOrStruct',...
                ['Expected argument %d to be a string property name ' ...
                'or an options structure\ncreated with OPTSET.'], i);
        end
        for j = 1:m
            if any(strcmp(fieldnames(arg),deblank(Names(j,:))))
                val = arg.(deblank(Names(j,:)));
            else
                val = [];
            end
            if ~isempty(val)
                options.(deblank(Names(j,:))) = val;
            end
        end
    end
    i = i + 1;
end

% A finite state machine to parse name-value pairs.
if rem(nargin-i+1,2) ~= 0
    error('optset:ArgNameValueMismatch',...
        'Arguments must occur in name-value pairs.');
end
expectval = 0;                          % start expecting a name, not a value
while i <= nargin
    arg = varargin{i};

    if ~expectval
        if ~ischar(arg)
            error('optset:NoPropName',...
                'Expected argument %d to be a string property name.', i);
        end

        lowArg = lower(arg);
        j = strmatch(lowArg,names);
        if isempty(j)                       % if no matches
            error('optset:InvalidPropName',...
                'Unrecognized property name ''%s''.', arg);
        elseif length(j) > 1                % if more than one match
            % Check for any exact matches (in case any names are subsets of others)
            k = strmatch(lowArg,names,'exact');
            if length(k) == 1
                j = k;
            else
                msg = sprintf('Ambiguous property name ''%s'' ', arg);
                msg = [msg '(' deblank(Names(j(1),:))];
                for k = j(2:length(j))'
                    msg = [msg ', ' deblank(Names(k,:))];
                end
                msg = sprintf('%s).', msg);
                error('optset:AmbiguousPropName', msg);
            end
        end
        expectval = 1;                      % we expect a value next

    else
        options.(deblank(Names(j,:))) = arg;
        expectval = 0;

    end
    i = i + 1;
end

if expectval
    error('optset:NoValueForProp',...
        'Expected value for property ''%s''.', arg);
end